package org.bench4q.agent.test.scenario;

import static org.junit.Assert.*;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;
import java.util.Stack;

import org.bench4q.agent.scenario.instruction.ControlInstruction;
import org.bench4q.agent.scenario.instruction.Instruction;
import org.bench4q.agent.scenario.instruction.SampleInstruction;
import org.bench4q.agent.scenario.instruction.TestInstruction;
import org.bench4q.agent.scenario.instruction.TimerInstruction;
import org.bench4q.agent.scenario.node.Behavior;
import org.bench4q.agent.scenario.node.PluginImplementBehavior;
import org.bench4q.share.models.agent.ParameterModel;
import org.bench4q.share.models.agent.scriptrecord.BehaviorModel;
import org.junit.Test;

public class Test_Behavior {
	@Test
	public void tsetComileUserBehaviorToInstructions() {
		PluginImplementBehavior behavior = (PluginImplementBehavior) Behavior
				.buildWith(BehaviorModel.UserBehaviorBuilder(1, "GET", "Http",
						Arrays.asList(ParameterModel.createParameter("url",
								"http://www.baidu.com"))));
		ArrayList<Instruction> code = new ArrayList<Instruction>();
		Stack<TestInstruction> conditions = new Stack<TestInstruction>();
		behavior.compile(code, conditions);
		assertEquals(1, code.size());
		assertEquals(0, conditions.size());
		assertEquals(behavior.getName(),
				((SampleInstruction) code.get(0)).getBehaviorName());
		assertEquals(behavior.getUse(),
				((SampleInstruction) code.get(0)).getPluginName());
		assertEquals(1,
				((SampleInstruction) code.get(0)).getParameters().length);
		assertArrayEquals(behavior.getParameters(),
				((SampleInstruction) code.get(0)).getParameters());
	}

	@Test
	public void testCompileTimerBehaviorToInstruction() {
		PluginImplementBehavior behavior = (PluginImplementBehavior) Behavior
				.buildWith(BehaviorModel.TimerBehaviorBuilder(1, "sleep",
						"ConstantTimer", Arrays.asList(ParameterModel
								.createParameter("time", "20"))));
		ArrayList<Instruction> code = new ArrayList<Instruction>();
		Stack<TestInstruction> conditions = new Stack<TestInstruction>();
		behavior.compile(code, conditions);
		assertEquals(1, code.size());
		assertEquals(0, conditions.size());
		assertEquals(behavior.getName(),
				((TimerInstruction) code.get(0)).getBehaviorName());
		assertEquals(behavior.getUse(),
				((TimerInstruction) code.get(0)).getPluginName());
		assertEquals(1, ((TimerInstruction) code.get(0)).getParameters().length);
		assertArrayEquals(behavior.getParameters(),
				((TimerInstruction) code.get(0)).getParameters());
	}

	@Test
	public void testCompileControlBehavior2Instruction() {
		PluginImplementBehavior behavior = (PluginImplementBehavior) Behavior
				.buildWith(BehaviorModel.ControlBehaviorBuilder(1, "next",
						"CsvProvider", Collections.<ParameterModel> emptyList()));
		ArrayList<Instruction> code = new ArrayList<Instruction>();
		Stack<TestInstruction> conditions = new Stack<TestInstruction>();
		behavior.compile(code, conditions);
		assertEquals(1, code.size());
		assertEquals(0, conditions.size());
		assertEquals(behavior.getName(),
				((ControlInstruction) code.get(0)).getBehaviorName());
		assertEquals(behavior.getUse(),
				((ControlInstruction) code.get(0)).getPluginName());
		assertEquals(0,
				((ControlInstruction) code.get(0)).getParameters().length);
		assertArrayEquals(behavior.getParameters(),
				((ControlInstruction) code.get(0)).getParameters());
	}

	@Test
	public void testCompileIfBehavior2Instruction() {
		// TODO:
		List<BehaviorModel> elseBehaviors = new ArrayList<BehaviorModel>(), thenBehaviors = new ArrayList<BehaviorModel>();
		Behavior behavior = Behavior.buildWith(BehaviorModel.IfBehaviorBuilder(
				BehaviorModel.testBehaviorBuilder(1, "isDefined", "context",
						Arrays.asList(ParameterModel.createParameter("name",
								"a"))), thenBehaviors, elseBehaviors));
		assertNotNull(behavior);
	}
}
