package org.bench4q.master.domain.valueobject.datastatistics.impl;

import org.bench4q.master.domain.valueobject.datastatistics.DataStatistics;
import org.bench4q.share.models.agent.statistics.BehaviorStatusCodeResultModel;
import org.bench4q.share.models.agent.statistics.DataStatisticsModel;
import org.bench4q.share.models.master.statistics.ScriptBehaviorStatusCodeModel;

public class BehaviorStatusCodeStatistics implements DataStatistics {
	private final int behaviorId;
	private final int statusCode;
	private final String contentType;
	private long countThisTime;
	private long countFromBegin;
	private long contentLengthFromBegin;
	private long contentLengthThisTime;
	private long minResponseTime;
	private long maxResponseTime;
	private long totalResponseThisTime;

	public BehaviorStatusCodeStatistics(int behaviorId, int statusCode,
			String contentType) {
		this.behaviorId = behaviorId;
		this.statusCode = statusCode;
		this.contentType = contentType;
		resetTemperaryField();
	}

	public void add(DataStatisticsModel dataUnit) {
		if (dataUnit == null
				|| !(dataUnit instanceof BehaviorStatusCodeResultModel)) {
			return;
		}
		BehaviorStatusCodeResultModel statusCodeResult = (BehaviorStatusCodeResultModel) dataUnit;
		this.countThisTime++;
		this.countFromBegin++;
		this.contentLengthFromBegin += statusCodeResult.getContentLength();
		this.contentLengthThisTime += statusCodeResult
				.getContentLengthThisTime();
		this.maxResponseTime = Math.max(this.maxResponseTime,
				statusCodeResult.getMaxResponseTime());
		this.minResponseTime = Math.min(this.minResponseTime,
				statusCodeResult.getMinResponseTime());
		this.totalResponseThisTime += statusCodeResult
				.getTotalResponseTimeThisTime();
	}

	/**
	 * Now, for totalCount, i just collect all result from agent, and add their
	 * countFromBegin together. But there will be a error there. if an agent
	 * doesn't give back its result, the the countFromBegin maybe less that last
	 * one.
	 */
	private void resetTemperaryField() {
		this.totalResponseThisTime = 0;
		this.countFromBegin = 0;
		this.countThisTime = 0;
		this.maxResponseTime = Long.MIN_VALUE;
		this.minResponseTime = Long.MAX_VALUE;
		this.contentLengthFromBegin = 0;
		this.contentLengthThisTime = 0;
	}

	public Object getStatistics() {
		ScriptBehaviorStatusCodeModel result = new ScriptBehaviorStatusCodeModel();
		result.setBehaviorId(this.behaviorId);
		result.setContentLength(this.contentLengthFromBegin);
		result.setContentType(this.contentType);
		result.setCount(this.countFromBegin);
		result.setMaxResponseTime(this.maxResponseTime);
		result.setMinResponseTime(this.minResponseTime);
		result.setStatusCode(this.statusCode);
		result.setAverageResponseTime(this.countThisTime == 0 ? 0
				: this.totalResponseThisTime / this.countThisTime);
		result.setContentLength(this.contentLengthFromBegin);
		result.setContentLengthThisTime(this.contentLengthThisTime);
		resetTemperaryField();
		return result;
	}

}
